#include "ehs/io/mdl/MdlCodec.h"

namespace ehs
{
	MdlCodec::MdlCodec()
		: hashExt(0), endianness(Endianness::LE), encoder(nullptr), decoder(nullptr)
	{
	}

	MdlCodec::MdlCodec(Str_8 id, Str_8 ext, const Endianness end, EnocdeMdlCb encoder, DecodeMdlCb decoder)
		: id(std::move(id)), hashExt(ext.Hash_64()), ext(std::move(ext)), endianness(end), encoder(encoder),
		decoder(decoder)
	{
	}

	MdlCodec::MdlCodec(MdlCodec&& codec) noexcept
		: id(std::move(codec.id)), hashExt(codec.hashExt), ext(std::move(codec.ext)), endianness(codec.endianness),
		encoder(codec.encoder), decoder(codec.decoder)
	{
		codec.hashExt = 0;
		codec.endianness = Endianness::LE;
		codec.encoder = nullptr;
		codec.decoder = nullptr;
	}

	MdlCodec::MdlCodec(const MdlCodec& codec)
		: id(codec.id), hashExt(codec.hashExt), ext(codec.ext), endianness(codec.endianness), encoder(codec.encoder),
		decoder(codec.decoder)
	{
	}

	MdlCodec& MdlCodec::operator=(MdlCodec&& codec) noexcept
	{
		if (this == &codec)
			return *this;

		id = std::move(codec.id);
		hashExt = codec.hashExt;
		ext = std::move(codec.ext);
		endianness = codec.endianness;
		encoder = codec.encoder;
		decoder = codec.decoder;

		codec.hashExt = 0;
		codec.endianness = Endianness::LE;
		codec.encoder = nullptr;
		codec.decoder = nullptr;

		return *this;
	}

	MdlCodec& MdlCodec::operator=(const MdlCodec& codec)
	{
		if (this == &codec)
			return *this;

		id = codec.id;
		hashExt = codec.hashExt;
		ext = codec.ext;
		endianness = codec.endianness;
		encoder = codec.encoder;
		decoder = codec.decoder;

		return *this;
	}

	Str_8 MdlCodec::GetId() const
	{
		return id;
	}

	UInt_64 MdlCodec::GetHashExt() const
	{
		return hashExt;
	}

	Str_8 MdlCodec::GetExt() const
	{
		return ext;
	}

	Endianness MdlCodec::GetEndianness() const
	{
		return endianness;
	}

	bool MdlCodec::Encode(Serializer<UInt_64>& out, const Mdl* in) const
	{
		if (!encoder)
		{
			EHS_LOG_INT("Error", 0, "Encoding is not supported for the " + id + " format.");
			return false;
		}

		return encoder(this, out, in);
	}

	bool MdlCodec::Decode(Serializer<UInt_64>& in, Mdl* out) const
	{
		if (!decoder)
		{
			EHS_LOG_INT("Error", 0, "Decoding is not supported for the " + id + " format.");
			return false;
		}

		return decoder(this, in, out);
	}
}